//---------------------------------------------------------------------------
#ifndef PGPbuffersH
#define PGPbuffersH

#include "mixmaster.h"
#include "fileman.h"
//---------------------------------------------------------------------------
#define ENABLE true
#define REWIND true

class TPGPbuffer : public TBUFFER
{
public:
   virtual __fastcall TPGPbuffer();
   virtual __fastcall TPGPbuffer(AnsiString keyringfile);
   virtual __fastcall TPGPbuffer(TStrings *asciiblock);
   virtual __fastcall TPGPbuffer(TPGPbuffer *buf);
   virtual __fastcall~TPGPbuffer();
   bool __fastcall copyfrom(TPGPbuffer *buffer, int length=0);
   void __fastcall import(TStrings *asciiblock);
   void __fastcall export(TStrings *asciiblock);
   bool __fastcall copypacketfrom(TPGPbuffer *pkts, bool rewind=false);
   bool __fastcall copykeyfrom(TPGPbuffer *keys, bool rewind=false); // sets ptr to 0 when done, ready to process key
   bool __fastcall appendkeyfrom(TPGPbuffer *keys, bool rewind=false);
   bool __fastcall enable(bool enable);
   bool __fastcall save();
   void __fastcall getdescrips(TStrings *lines);
   bool __fastcall keyexists(TPGPbuffer *key);
   AnsiString __fastcall getdescrips();
   bool __fastcall get(AnsiString IdText8, TPGPbuffer *key);
   int  __fastcall keyid(TPGPbuffer *keyid);
   // properties
   __property byte CTB                          = { read=GetCTB };
   __property bool enabled                      = { read=Enabled };
   __property AnsiString idtext8                = { read=IdText8 };
   __property AnsiString firstuid               = { read=FirstUID };
   __property byte version                      = { read=Version };
   __property bool ispair                       = { read=IsPair };
   __property int count                         = { read=Count };
   __property bool eok                          = { read=EndOfKey };

   // older routines using upper and lower case
   int  __fastcall GetPacketSize(); // whole pktlen, ptr at CTB
   int  __fastcall GetPacketInfo(byte *CTB, byte *lolf=NULL);// datlen & CTB, ptr at data
   //int  __fastcall KeyId(TPGPbuffer *keyid);
   //bool __fastcall Enable(bool enable);
   //bool __fastcall Save();

   virtual byte __fastcall ReadByte();
   virtual int  __fastcall ReadInt();          // reads 2 bytes not 4
   virtual long __fastcall ReadLong();
   virtual int  __fastcall WriteByte(byte b);
   virtual int  __fastcall WriteInt(int i);    // writes 2 bytes not 4
   virtual int  __fastcall WriteLong(long l);
   byte __fastcall GetCTB();
   bool __fastcall EndOfKey(); // is the buf ptr on an eok byte
   AnsiString __fastcall FirstUID();
   AnsiString __fastcall IdText8();
   bool __fastcall Enabled();
   bool __fastcall IsPair();
   byte __fastcall Version();
   int  __fastcall Count();
};
// packet types
#define PublicKeyPacket(CTB)    (CTB==153) //((CTB&152)==152)
#define PublicSubKeyPacket(CTB) (CTB==185) //((CTB&184)==184)
#define SecretKeyPacket(CTB)    (CTB==149) //((CTB&148)==148)
#define SignaturePacket(CTB)    (CTB==137)
#define UserIdPacket(CTB)       (CTB==180) //((CTB&180)==180)
#define TrustPacket(CTB)        (CTB==176) //((CTB&176)==176)
// algorithms
#define PGP_ANY 0
#define PGP_ES_RSA 1
#define PGP_E_ELG 16
#define PGP_S_DSA 17
// SHA
#define SHA_DIGEST_LENGTH 20
// SHA in libeay.dll
void __fastcall SHA1ex(byte *b, int n, byte* m);
typedef void (*_SHA1ex)(byte*, int, byte*);
// misc pgp support functions
int __fastcall pgparmor(BUFFER *in, int mode);
int __fastcall pgpdearmor(BUFFER *in, BUFFER *out);
int __fastcall pgp_ispacketex(BUFFER *b);
int __fastcall pgp_nummpiex(int algo);
int __fastcall digestmem_sha1ex(byte *b, int n, BUFFER *md);
int __fastcall digest_sha1ex(BUFFER *b, BUFFER *md);
int __fastcall mpi_putex(BUFFER *buf, BUFFER *mpi);
int __fastcall mpi_getex(BUFFER *b, BUFFER *mpi);
#define begin_pgp "-----BEGIN PGP "
#define begin_pgpmsg "-----BEGIN PGP MESSAGE-----"
#define begin_pgpkey "-----BEGIN PGP PUBLIC KEY BLOCK-----"
#define begin_pgpsigned "-----BEGIN PGP SIGNED MESSAGE-----"
#define begin_pgpsig "-----BEGIN PGP SIGNATURE-----"
#define end_pgp "-----END PGP "
#define end_pgpmsg "-----END PGP MESSAGE-----"
#define end_pgpkey "-----END PGP PUBLIC KEY BLOCK-----"
#define end_pgpsig "-----END PGP SIGNATURE-----"
#endif
