//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "PgpKeyImptDlgs.h"
#include "PgpFiles.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TPgpKeyImptDlg *PgpKeyImptDlg;
const AnsiString PgpNoKeysFoundAtAll = "...NO KEYS FOUND AT ALL!";
const AnsiString PgpLoading = "Loading...";
//---------------------------------------------------------------------------
bool __fastcall ImportPGPKeys(TForm *Owner, TPGPbuffer *ImpKeys, AnsiString &ImportDir)
{
   PgpKeyImptDlg = new TPgpKeyImptDlg(Owner,ImpKeys,ImportDir);
   PgpKeyImptDlg->ShowModal();
   bool ok = PgpKeyImptDlg->ModalResult==mrOk && ImpKeys->count;
   delete PgpKeyImptDlg;
   ImpKeys->rewind();
   return ok;
}
//---------------------------------------------------------------------------
__fastcall TPgpKeyImptDlg::TPgpKeyImptDlg(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TPgpKeyImptDlg::TPgpKeyImptDlg(TComponent* Owner, TPGPbuffer *ImpKeys,
   AnsiString &ImportDir) : TForm(Owner)
{
   imports = ImpKeys;
   imports->clear();
   keys = new TPGPbuffer();

   CenterDialog(this);
   impdir = &ImportDir;
   OpenDialog->InitialDir = *impdir;
   OpenDialog->Filter = "PGP import files (*.asc)|*.asc|All files (*.*)|*.*";
   OpenDialog->DefaultExt = ".asc";
   OpenDialog->FileName = "";//"*.asc";
   ComboBox1->Items->Text = "Import Personal PGP Keys";
   ComboBox1->ItemIndex = 0;
   cap = Caption;
}
//---------------------------------------------------------------------------
__fastcall TPgpKeyImptDlg::~TPgpKeyImptDlg()
{
   for (int i=0; i<ListBox1->Items->Count; i++)
      delete dynamic_cast<TPGPbuffer*>(ListBox1->Items->Objects[i]);
   delete keys;
}
//---------------------------------------------------------------------------
TPGPbuffer* __fastcall TPgpKeyImptDlg::IsPublic(TPGPbuffer *key)
{
   TPGPbuffer *pub=NULL;
   AnsiString id8,keyid8=key->idtext8;
   for (int i=0; i<ListBox1->Items->Count; i++)
   {
      id8 = ListBox1->Items->Strings[i].SubString(3,8);
      if (id8==keyid8)
      {
         pub=dynamic_cast<TPGPbuffer*>(ListBox1->Items->Objects[i]);
         break;
      }
   }
   return pub;
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImptDlg::ImportKeys(AnsiString Name, TStrings *AsciiBlock)
{
   Caption = "";
   keys->clear();
   ListBox1->Clear();

   ImportBtn->Enabled = false;
   AnsiString cshort,curdesc,desc,keytext,orglines=AsciiBlock->Text;
   keys->AddAscii(AsciiBlock);

   SetRedraw(Panel1,0);
   int newcnt = keys->count;
   if (newcnt)
   {
      keys->rewind();
      TPGPbuffer *pubkey,*key = new TPGPbuffer();
      while (key->CopyKeyFrom(keys,true))
      {
         if (SecretKeyPacket(key->CTB) && (pubkey=IsPublic(key))!=NULL)
            pubkey->AppendKeyFrom(key); // add secret key to end of public mate
         else
         if (PublicKeyPacket(key->CTB) && !IsPublic(key)) // add public to list
            ListBox1->Items->AddObject("0x"+key->idtext8+"  "+key->firstuid,
               new TPGPbuffer(key));
         key->clear();
      }
      // select the entire list
      for (int i=0; i<ListBox1->Items->Count; i++)
      {
         ListBox1->Selected[i] = true;
        ((TPGPbuffer*)ListBox1->Items->Objects[i])->rewind();
      }
      ListBox1->TopIndex = 0;
      delete key;
      ImportBtn->Enabled = true;
   }
   else
   {
      ListBox1->Items->Add(Name+PgpNoKeysFoundAtAll);
      ListBox1->Items->Add("Done");
   }
   while (flash<4) AppProcessMessages();
   Timer1->Enabled = false;

   AppProcessMessages();
   SetRedraw(Panel1,1);
   Caption = cap+" ..."+s(newcnt)+" key"+(newcnt!=1?"s":"");
   MyBeep(MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImptDlg::ClipBtnClick(TObject *Sender)
{
   if (MsgBoxHwnd(Handle,"Import keys from the clipboard?","Confirm",MB_OKCANCEL)==IDOK)
   {
      ListBox1->Clear();
      ListBox1->BringToFront();
      RichEdit1->Clear();
      TStringList *clip = new TStringList();
      PasteFromClipboard(clip);

      flash=0; Timer1->Enabled = true;
      ImportKeys("Clipboard",clip);
      Timer1->Enabled = false;
      ActiveControl = Panel1;

      if (ListBox1->Items->Count &&
          ListBox1->Items->Strings[0].Pos(PgpNoKeysFoundAtAll)>0)
      {
         RichEdit1->Lines->Add("Clipboard"+PgpNoKeysFoundAtAll);
         RichEdit1->BringToFront();
         RichEdit1->Lines->Add("");
         RichEdit1->Lines->Add("Clipboard Contents-------------------------------------");
         RichEdit1->Lines->AddStrings(clip);
      }
      delete clip;
   }
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImptDlg::FileBtnClick(TObject *Sender)
{
   if (OpenDialog->Execute())
   {
      ListBox1->Clear();
      RichEdit1->Clear();
      ListBox1->BringToFront();

      flash=0; Timer1->Enabled = true;
      TStringList *keys = new TStringList();
      keys->LoadFromFile(OpenDialog->FileName);
      ImportKeys(ExtractFileName(OpenDialog->FileName),keys);
      Timer1->Enabled = false;

      if (ListBox1->Items->Count &&
          ListBox1->Items->Strings[0].Pos(PgpNoKeysFoundAtAll)>0)
      {
         RichEdit1->Lines->Add(OpenDialog->FileName+PgpNoKeysFoundAtAll);
         RichEdit1->BringToFront();
         RichEdit1->Lines->Add("");
         RichEdit1->Lines->Add(OpenDialog->FileName+" Contents-----------------------------------------------");
         RichEdit1->Lines->AddStrings(keys);
      }

     *impdir = ExtractFilePath(OpenDialog->FileName);
      OpenDialog->InitialDir = *impdir;
      ActiveControl = Panel1;
      delete keys;
   }
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImptDlg::Timer1Timer(TObject *Sender)
{
   if (Caption.IsEmpty()) Caption = PgpLoading;
   else Caption = "";
   flash++;
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImptDlg::ImportBtnClick(TObject *Sender)
{
   if (MsgBoxHwnd(Handle,"Import selected keys now?","Confirm",MB_OKCANCEL)==IDOK)
   {
      imports->clear();
      for (int i=0; i<ListBox1->Items->Count; i++)
      {
         if (ListBox1->Selected[i])
         {  // public keys are in individual buffers in items array
            // some only public, some with trailing privates
            TPGPbuffer *key=(TPGPbuffer*)ListBox1->Items->Objects[i];
            imports->CopyFrom(key,key->length);
         }
      }
      ModalResult = mrOk;
      imports->rewind();
   }
}
//---------------------------------------------------------------------------
