//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
//#include <pgpEncode.h>
//#include <pgpoptionlist.h>
//#include <pgpPubTypes.h>
#include <pgpKeys.h>
#include <pgpErrors.h>
#include <pgpUtilities.h>
#include <pgpSDKPrefs.h>
#include <pgpFeatures.h>
#include "QSpgp5ver.h"
/*
* SDK API version
* example: 1.0.0 = 0x01000000
*   0x01000000   (16777216)   SDK 1.0.0   PGP550
*   0x02000000   (33554432)   SDK 1.1.0
*   0x02000010   (33554448)   SDK 1.1.1   PGP553
*   0x03000000   (50331648)   SDK 1.5.0   PGP600
*   0x03000010   (50331664)   SDK 1.5.1
*   0x03000020   (50331680)   SDK 1.5.2   PGP602
*   0x03001000   (50335744)   SDK 1.6.0
*   0x03002000   (50339840)   SDK 1.7.0   PGP650
*   0x03002010   (50339856)   SDK 1.7.1   PGP651,2,3
*/
USEUNIT("QSpgp5ver.cpp");
USERES("QSpgp5.res");
USELIB("pgpsdk.lib");
//---------------------------------------------------------------------------
extern "C" __declspec(dllexport) bool __fastcall PGPqdkInit(HWND owner, HWND apphandle);
extern "C" __declspec(dllexport) bool __fastcall PGPqdkCopyCompat5(HWND owner,
   char *pubring, char *secring);
extern "C" __declspec(dllexport) UINT __fastcall PGPqdkSdkApiVersion();
extern "C" __declspec(dllexport) char*__fastcall PGPqdkPubringpkr();
extern "C" __declspec(dllexport) char*__fastcall PGPqdkSecringskr();
extern "C" __declspec(dllexport) void __fastcall PGPqdkCleanup();
bool __fastcall IsPgpError(PGPError err, char *errmsg);

HWND owner;
PGPError err;
PGPUInt32 apiversion;
PGPContextRef context;
char buffer[256],*pubringpkr,*secringskr;
#define createflag kPGPKeyRingOpenFlags_Create
#define muteflag kPGPKeyRingOpenFlags_Mutable
#define noflags false
//---------------------------------------------------------------------------
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void*)
{
  return 1;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool __fastcall PGPqdkInit(HWND Owner, HWND apphandle)
{
   PGPFileSpecRef pubSpec = kInvalidPGPFileSpecRef;
   PGPFileSpecRef secSpec = kInvalidPGPFileSpecRef;
   Application->Handle = apphandle;
   owner = Owner;

   err=PGPsdkInit();
   if (IsPgpError(err,"Unable to initialize PGPsdk!"))
      return false;

   err=PGPNewContext(kPGPsdkAPIVersion,&context);
   if (IsPgpError(err,"Failed to initialize PGP context structure!"))
      return false;

   err=PGPsdkLoadDefaultPrefs(context);
   if (IsPgpError(err,"Unable to load PGPsdk default preferences!"))
      return false;

   // if first attempt fails, use prefSelector from earlier sdk version
   err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)5,&pubSpec);
   if (err==-12000) err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)0,&pubSpec);
   if (IsPgpError(err,"Unable to get PGP public keyring file spec!"))
      goto exit;

   err=PGPGetFullPathFromFileSpec(pubSpec,&pubringpkr);
   if (IsPgpError(err,"Unable to get PGP public keyring path!"))
      goto exit;

   // if first attempt fails, use prefSelector from earlier sdk version
   err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)6,&secSpec);
   if (err==-12000) err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)1,&secSpec);
   if (IsPgpError(err,"Unable to get PGP secret keyring file spec!"))
      goto exit;

   err=PGPGetFullPathFromFileSpec(secSpec,&secringskr);
   if (IsPgpError(err,"Unable to get PGP secret keyring path!"))
      goto exit;

   err=PGPGetSDKVersion(&apiversion);
   if (IsPgpError(err,"Failed to get PGPsdk API version!"))
      goto exit;

   StrLower(pubringpkr);
   StrLower(secringskr);

exit:
   if (PGPFileSpecRefIsValid(secSpec)) PGPFreeFileSpec(secSpec);
   if (PGPFileSpecRefIsValid(pubSpec)) PGPFreeFileSpec(pubSpec);
   return IsntPGPError(err);
}
//---------------------------------------------------------------------------
void __fastcall PGPqdkCleanup()
{
   if (secringskr) PGPFreeData(secringskr);
   if (pubringpkr) PGPFreeData(pubringpkr);
   if (PGPContextRefIsValid(context)) PGPFreeContext(context);
   PGPsdkCleanup();
}
//---------------------------------------------------------------------------
char* __fastcall PGPqdkPubringpkr()
{
   return pubringpkr;
}
//---------------------------------------------------------------------------
char* __fastcall PGPqdkSecringskr()
{
   return secringskr;
}
//---------------------------------------------------------------------------
bool __fastcall PGPqdkCopyCompat5(HWND Owner, char *pubring, char *secring)
{
   return true;
}
//---------------------------------------------------------------------------
void __fastcall ShowPGPError(PGPError err, AnsiString errmsg)
{
	 if(IsPGPError(err))
   {
  	 char *buf = new char[256];
 	    PGPGetErrorString(err, 255, buf);
      MessageBox(owner,
         AnsiString(errmsg+"\nError: ("+AnsiString(err)+") "+AnsiString(buf)).c_str(),
        "PGP Error",
         MB_OK
      );
      delete buf;
   }
}
//---------------------------------------------------------------------------
bool __fastcall IsPgpError(PGPError err, char *errmsg)
{
   if (IsPGPError(err))
   {
      ShowPGPError(err,errmsg);
      return true;
   }
   return false;
}
//---------------------------------------------------------------------------
